@@ -1,9 +1,12 @@
+use v5.10;
 use strict;
 use warnings;
 package CPAN::Meta::Requirements;
 # ABSTRACT: a set of version requirements for a CPAN dist
 
-our $VERSION = '2.129';
+our $VERSION = '2.142';
+
+use CPAN::Meta::Requirements::Range;
 
 #pod =head1 SYNOPSIS
 #pod
@@ -23,7 +26,7 @@ our $VERSION = '2.129';
 #pod
 #pod A CPAN::Meta::Requirements object models a set of version constraints like
 #pod those specified in the F<META.yml> or F<META.json> files in CPAN distributions,
-#pod and as defined by L<CPAN::Meta::Spec>;
+#pod and as defined by L<CPAN::Meta::Spec>.
 #pod It can be built up by adding more and more constraints, and it will reduce them
 #pod to the simplest representation.
 #pod
@@ -33,23 +36,6 @@ our $VERSION = '2.129';
 #pod =cut
 
 use Carp ();
-use Scalar::Util ();
-
-# To help ExtUtils::MakeMaker bootstrap CPAN::Meta::Requirements on perls
-# before 5.10, we fall back to the EUMM bundled compatibility version module if
-# that's the only thing available.  This shouldn't ever happen in a normal CPAN
-# install of CPAN::Meta::Requirements, as version.pm will be picked up from
-# prereqs and be available at runtime.
-
-BEGIN {
-  eval "use version ()"; ## no critic
-  if ( my $err = $@ ) {
-    eval "use ExtUtils::MakeMaker::version" or die $err; ## no critic
-  }
-}
-
-# Perl 5.10.0 didn't have "is_qv" in version.pm
-*_is_qv = version->can('is_qv') ? sub { $_[0]->is_qv } : sub { exists $_[0]->{qv} };
 
 #pod =method new
 #pod
@@ -60,8 +46,9 @@ BEGIN {
 #pod
 #pod =for :list
 #pod * C<bad_version_hook> -- if provided, when a version cannot be parsed into
-#pod   a version object, this code reference will be called with the invalid version
-#pod   string as an argument.  It must return a valid version object.
+#pod   a version object, this code reference will be called with the invalid
+#pod   version string as first argument, and the module name as second
+#pod   argument.  It must return a valid version object.
 #pod
 #pod All other keys are ignored.
 #pod
@@ -79,67 +66,6 @@ sub new {
   return bless \%self => $class;
 }
 
-# from version::vpp
-sub _find_magic_vstring {
-  my $value = shift;
-  my $tvalue = '';
-  require B;
-  my $sv = B::svref_2object(\$value);
-  my $magic = ref($sv) eq 'B::PVMG' ? $sv->MAGIC : undef;
-  while ( $magic ) {
-    if ( $magic->TYPE eq 'V' ) {
-      $tvalue = $magic->PTR;
-      $tvalue =~ s/^v?(.+)$/v$1/;
-      last;
-    }
-    else {
-      $magic = $magic->MOREMAGIC;
-    }
-  }
-  return $tvalue;
-}
-
-sub _version_object {
-  my ($self, $version) = @_;
-
-  my $vobj;
-
-  # hack around version::vpp not handling <3 character vstring literals
-  if ( $INC{'version/vpp.pm'} || $INC{'ExtUtils/MakeMaker/version/vpp.pm'} ) {
-    my $magic = _find_magic_vstring( $version );
-    $version = $magic if length $magic;
-  }
-
-  eval {
-    local $SIG{__WARN__} = sub { die "Invalid version: $_[0]" };
-    $vobj  = (! defined $version)                ? version->new(0)
-           : (! Scalar::Util::blessed($version)) ? version->new($version)
-           :                                       $version;
-  };
-
-  if ( my $err = $@ ) {
-    my $hook = $self->{bad_version_hook};
-    $vobj = eval { $hook->($version) }
-      if ref $hook eq 'CODE';
-    unless (Scalar::Util::blessed($vobj) && $vobj->isa("version")) {
-      $err =~ s{ at .* line \d+.*$}{};
-      die "Can't convert '$version': $err";
-    }
-  }
-
-  # ensure no leading '.'
-  if ( $vobj =~ m{\A\.} ) {
-    $vobj = version->new("0$vobj");
-  }
-
-  # ensure normal v-string form
-  if ( _is_qv($vobj) ) {
-    $vobj = version->new($vobj->normal);
-  }
-
-  return $vobj;
-}
-
 #pod =method add_minimum
 #pod
 #pod   $req->add_minimum( $module => $version );
@@ -193,32 +119,62 @@ sub _version_object {
 #pod =cut
 
 BEGIN {
-  for my $type (qw(minimum maximum exclusion exact_version)) {
+  for my $type (qw(maximum exclusion exact_version)) {
     my $method = "with_$type";
     my $to_add = $type eq 'exact_version' ? $type : "add_$type";
 
     my $code = sub {
       my ($self, $name, $version) = @_;
 
-      $version = $self->_version_object( $version );
-
       $self->__modify_entry_for($name, $method, $version);
 
       return $self;
     };
-    
+
     no strict 'refs';
     *$to_add = $code;
   }
 }
 
+# add_minimum is optimized compared to generated subs above because
+# it is called frequently and with "0" or equivalent input
+sub add_minimum {
+  my ($self, $name, $version) = @_;
+
+  # stringify $version so that version->new("0.00")->stringify ne "0"
+  # which preserves the user's choice of "0.00" as the requirement
+  if (not defined $version or "$version" eq '0') {
+    return $self if $self->__entry_for($name);
+    Carp::croak("can't add new requirements to finalized requirements")
+      if $self->is_finalized;
+
+    $self->{requirements}{ $name } =
+      CPAN::Meta::Requirements::Range->with_minimum('0', $name);
+  }
+  else {
+    $self->__modify_entry_for($name, 'with_minimum', $version);
+  }
+  return $self;
+}
+
+#pod =method version_range_for_module
+#pod
+#pod   $req->version_range_for_module( $another_req_object );
+#pod
+#pod =cut
+
+sub version_range_for_module {
+  my ($self, $module) = @_;
+  return $self->{requirements}{$module};
+}
+
 #pod =method add_requirements
 #pod
 #pod   $req->add_requirements( $another_req_object );
 #pod
-#pod This method adds all the requirements in the given CPAN::Meta::Requirements object
-#pod to the requirements object on which it was called.  If there are any conflicts,
-#pod an exception is thrown.
+#pod This method adds all the requirements in the given CPAN::Meta::Requirements
+#pod object to the requirements object on which it was called.  If there are any
+#pod conflicts, an exception is thrown.
 #pod
 #pod This method returns the requirements object.
 #pod
@@ -228,11 +184,8 @@ sub add_requirements {
   my ($self, $req) = @_;
 
   for my $module ($req->required_modules) {
-    my $modifiers = $req->__entry_for($module)->as_modifiers;
-    for my $modifier (@$modifiers) {
-      my ($method, @args) = @$modifier;
-      $self->$method($module => @args);
-    };
+    my $new_range = $req->version_range_for_module($module);
+    $self->__modify_entry_for($module, 'with_range', $new_range);
   }
 
   return $self;
@@ -258,10 +211,8 @@ sub add_requirements {
 sub accepts_module {
   my ($self, $module, $version) = @_;
 
-  $version = $self->_version_object( $version );
-
   return 1 unless my $range = $self->__entry_for($module);
-  return $range->_accepts($version);
+  return $range->accepts($version);
 }
 
 #pod =method clear_requirement
@@ -279,7 +230,7 @@ sub clear_requirement {
 
   return $self unless $self->__entry_for($module);
 
-  Carp::confess("can't clear requirements on finalized requirements")
+  Carp::croak("can't clear requirements on finalized requirements")
     if $self->is_finalized;
 
   delete $self->{requirements}{ $module };
@@ -295,7 +246,7 @@ sub clear_requirement {
 #pod the format described in L<CPAN::Meta::Spec> or undef if the given module has no
 #pod requirements. This should only be used for informational purposes such as error
 #pod messages and should not be interpreted or used for comparison (see
-#pod L</accepts_module> instead.)
+#pod L</accepts_module> instead).
 #pod
 #pod =cut
 
@@ -306,6 +257,25 @@ sub requirements_for_module {
   return $entry->as_string;
 }
 
+#pod =method structured_requirements_for_module
+#pod
+#pod   $req->structured_requirements_for_module( $module );
+#pod
+#pod This returns a data structure containing the version requirements for a given
+#pod module or undef if the given module has no requirements.  This should
+#pod not be used for version checks (see L</accepts_module> instead).
+#pod
+#pod Added in version 2.134.
+#pod
+#pod =cut
+
+sub structured_requirements_for_module {
+  my ($self, $module) = @_;
+  my $entry = $self->__entry_for($module);
+  return unless $entry;
+  return $entry->as_struct;
+}
+
 #pod =method required_modules
 #pod
 #pod This method returns a list of all the modules for which requirements have been
@@ -339,13 +309,13 @@ sub __modify_entry_for {
   my $fin = $self->is_finalized;
   my $old = $self->__entry_for($name);
 
-  Carp::confess("can't add new requirements to finalized requirements")
+  Carp::croak("can't add new requirements to finalized requirements")
     if $fin and not $old;
 
-  my $new = ($old || 'CPAN::Meta::Requirements::_Range::Range')
-          ->$method($version);
+  my $new = ($old || 'CPAN::Meta::Requirements::Range')
+          ->$method($version, $name, $self->{bad_version_hook});
 
-  Carp::confess("can't modify finalized requirements")
+  Carp::croak("can't modify finalized requirements")
     if $fin and $old->as_string ne $new->as_string;
 
   $self->{requirements}{ $name } = $new;
@@ -362,7 +332,7 @@ sub is_simple {
   my ($self) = @_;
   for my $module ($self->required_modules) {
     # XXX: This is a complete hack, but also entirely correct.
-    return if $self->__entry_for($module)->as_string =~ /\s/;
+    return if not $self->__entry_for($module)->is_simple;
   }
 
   return 1;
@@ -468,267 +438,39 @@ sub as_string_hash {
 #pod
 #pod =cut
 
-my %methods_for_op = (
-  '==' => [ qw(exact_version) ],
-  '!=' => [ qw(add_exclusion) ],
-  '>=' => [ qw(add_minimum)   ],
-  '<=' => [ qw(add_maximum)   ],
-  '>'  => [ qw(add_minimum add_exclusion) ],
-  '<'  => [ qw(add_maximum add_exclusion) ],
-);
-
 sub add_string_requirement {
   my ($self, $module, $req) = @_;
 
-  Carp::confess("No requirement string provided for $module")
-    unless defined $req && length $req;
-
-  my $magic = _find_magic_vstring( $req );
-  if (length $magic) {
-    $self->add_minimum($module => $magic);
-    return;
-  }
-
-  my @parts = split qr{\s*,\s*}, $req;
-
-  for my $part (@parts) {
-    my ($op, $ver) = $part =~ m{\A\s*(==|>=|>|<=|<|!=)\s*(.*)\z};
-
-    if (! defined $op) {
-      $self->add_minimum($module => $part);
-    } else {
-      Carp::confess("illegal requirement string: $req")
-        unless my $methods = $methods_for_op{ $op };
-
-      $self->$_($module => $ver) for @$methods;
-    }
-  }
+  $self->__modify_entry_for($module, 'with_string_requirement', $req);
 }
 
 #pod =method from_string_hash
 #pod
 #pod   my $req = CPAN::Meta::Requirements->from_string_hash( \%hash );
+#pod   my $req = CPAN::Meta::Requirements->from_string_hash( \%hash, \%opts );
 #pod
-#pod This is an alternate constructor for a CPAN::Meta::Requirements object.  It takes
-#pod a hash of module names and version requirement strings and returns a new
-#pod CPAN::Meta::Requirements object. As with add_string_requirement, a
-#pod version can be a Perl "v-string".
+#pod This is an alternate constructor for a CPAN::Meta::Requirements
+#pod object. It takes a hash of module names and version requirement
+#pod strings and returns a new CPAN::Meta::Requirements object. As with
+#pod add_string_requirement, a version can be a Perl "v-string". Optionally,
+#pod you can supply a hash-reference of options, exactly as with the L</new>
+#pod method.
 #pod
 #pod =cut
 
 sub from_string_hash {
-  my ($class, $hash) = @_;
+  my ($class, $hash, $options) = @_;
 
-  my $self = $class->new;
+  my $self = $class->new($options);
 
   for my $module (keys %$hash) {
     my $req = $hash->{$module};
-    unless ( defined $req && length $req ) {
-      $req = 0;
-      Carp::carp("Undefined requirement for $module treated as '0'");
-    }
     $self->add_string_requirement($module, $req);
   }
 
   return $self;
 }
 
-##############################################################
-
-{
-  package
-    CPAN::Meta::Requirements::_Range::Exact;
-  sub _new     { bless { version => $_[1] } => $_[0] }
-
-  sub _accepts { return $_[0]{version} == $_[1] }
-
-  sub as_string { return "== $_[0]{version}" }
-
-  sub as_modifiers { return [ [ exact_version => $_[0]{version} ] ] }
-
-  sub _clone {
-    (ref $_[0])->_new( version->new( $_[0]{version} ) )
-  }
-
-  sub with_exact_version {
-    my ($self, $version) = @_;
-
-    return $self->_clone if $self->_accepts($version);
-
-    Carp::confess("illegal requirements: unequal exact version specified");
-  }
-
-  sub with_minimum {
-    my ($self, $minimum) = @_;
-    return $self->_clone if $self->{version} >= $minimum;
-    Carp::confess("illegal requirements: minimum above exact specification");
-  }
-
-  sub with_maximum {
-    my ($self, $maximum) = @_;
-    return $self->_clone if $self->{version} <= $maximum;
-    Carp::confess("illegal requirements: maximum below exact specification");
-  }
-
-  sub with_exclusion {
-    my ($self, $exclusion) = @_;
-    return $self->_clone unless $exclusion == $self->{version};
-    Carp::confess("illegal requirements: excluded exact specification");
-  }
-}
-
-##############################################################
-
-{
-  package
-    CPAN::Meta::Requirements::_Range::Range;
-
-  sub _self { ref($_[0]) ? $_[0] : (bless { } => $_[0]) }
-
-  sub _clone {
-    return (bless { } => $_[0]) unless ref $_[0];
-
-    my ($s) = @_;
-    my %guts = (
-      (exists $s->{minimum} ? (minimum => version->new($s->{minimum})) : ()),
-      (exists $s->{maximum} ? (maximum => version->new($s->{maximum})) : ()),
-
-      (exists $s->{exclusions}
-        ? (exclusions => [ map { version->new($_) } @{ $s->{exclusions} } ])
-        : ()),
-    );
-
-    bless \%guts => ref($s);
-  }
-
-  sub as_modifiers {
-    my ($self) = @_;
-    my @mods;
-    push @mods, [ add_minimum => $self->{minimum} ] if exists $self->{minimum};
-    push @mods, [ add_maximum => $self->{maximum} ] if exists $self->{maximum};
-    push @mods, map {; [ add_exclusion => $_ ] } @{$self->{exclusions} || []};
-    return \@mods;
-  }
-
-  sub as_string {
-    my ($self) = @_;
-
-    return 0 if ! keys %$self;
-
-    return "$self->{minimum}" if (keys %$self) == 1 and exists $self->{minimum};
-
-    my @exclusions = @{ $self->{exclusions} || [] };
-
-    my @parts;
-
-    for my $pair (
-      [ qw( >= > minimum ) ],
-      [ qw( <= < maximum ) ],
-    ) {
-      my ($op, $e_op, $k) = @$pair;
-      if (exists $self->{$k}) {
-        my @new_exclusions = grep { $_ != $self->{ $k } } @exclusions;
-        if (@new_exclusions == @exclusions) {
-          push @parts, "$op $self->{ $k }";
-        } else {
-          push @parts, "$e_op $self->{ $k }";
-          @exclusions = @new_exclusions;
-        }
-      }
-    }
-
-    push @parts, map {; "!= $_" } @exclusions;
-
-    return join q{, }, @parts;
-  }
-
-  sub with_exact_version {
-    my ($self, $version) = @_;
-    $self = $self->_clone;
-
-    Carp::confess("illegal requirements: exact specification outside of range")
-      unless $self->_accepts($version);
-
-    return CPAN::Meta::Requirements::_Range::Exact->_new($version);
-  }
-
-  sub _simplify {
-    my ($self) = @_;
-
-    if (defined $self->{minimum} and defined $self->{maximum}) {
-      if ($self->{minimum} == $self->{maximum}) {
-        Carp::confess("illegal requirements: excluded all values")
-          if grep { $_ == $self->{minimum} } @{ $self->{exclusions} || [] };
-
-        return CPAN::Meta::Requirements::_Range::Exact->_new($self->{minimum})
-      }
-
-      Carp::confess("illegal requirements: minimum exceeds maximum")
-        if $self->{minimum} > $self->{maximum};
-    }
-
-    # eliminate irrelevant exclusions
-    if ($self->{exclusions}) {
-      my %seen;
-      @{ $self->{exclusions} } = grep {
-        (! defined $self->{minimum} or $_ >= $self->{minimum})
-        and
-        (! defined $self->{maximum} or $_ <= $self->{maximum})
-        and
-        ! $seen{$_}++
-      } @{ $self->{exclusions} };
-    }
-
-    return $self;
-  }
-
-  sub with_minimum {
-    my ($self, $minimum) = @_;
-    $self = $self->_clone;
-
-    if (defined (my $old_min = $self->{minimum})) {
-      $self->{minimum} = (sort { $b cmp $a } ($minimum, $old_min))[0];
-    } else {
-      $self->{minimum} = $minimum;
-    }
-
-    return $self->_simplify;
-  }
-
-  sub with_maximum {
-    my ($self, $maximum) = @_;
-    $self = $self->_clone;
-
-    if (defined (my $old_max = $self->{maximum})) {
-      $self->{maximum} = (sort { $a cmp $b } ($maximum, $old_max))[0];
-    } else {
-      $self->{maximum} = $maximum;
-    }
-
-    return $self->_simplify;
-  }
-
-  sub with_exclusion {
-    my ($self, $exclusion) = @_;
-    $self = $self->_clone;
-
-    push @{ $self->{exclusions} ||= [] }, $exclusion;
-
-    return $self->_simplify;
-  }
-
-  sub _accepts {
-    my ($self, $version) = @_;
-
-    return if defined $self->{minimum} and $version < $self->{minimum};
-    return if defined $self->{maximum} and $version > $self->{maximum};
-    return if defined $self->{exclusions}
-          and grep { $version == $_ } @{ $self->{exclusions} };
-
-    return 1;
-  }
-}
-
 1;
 # vim: ts=2 sts=2 sw=2 et:
 
@@ -744,7 +486,7 @@ CPAN::Meta::Requirements - a set of version requirements for a CPAN dist
 
 =head1 VERSION
 
-version 2.129
+version 2.142
 
 =head1 SYNOPSIS
 
@@ -764,7 +506,7 @@ version 2.129
 
 A CPAN::Meta::Requirements object models a set of version constraints like
 those specified in the F<META.yml> or F<META.json> files in CPAN distributions,
-and as defined by L<CPAN::Meta::Spec>;
+and as defined by L<CPAN::Meta::Spec>.
 It can be built up by adding more and more constraints, and it will reduce them
 to the simplest representation.
 
@@ -784,7 +526,7 @@ hash reference argument.  Currently, only one key is supported:
 
 =item *
 
-C<bad_version_hook> -- if provided, when a version cannot be parsed into a version object, this code reference will be called with the invalid version string as an argument.  It must return a valid version object.
+C<bad_version_hook> -- if provided, when a version cannot be parsed into a version object, this code reference will be called with the invalid version string as first argument, and the module name as second argument.  It must return a valid version object.
 
 =back
 
@@ -840,13 +582,17 @@ version.  No other version would be considered acceptable.
 
 This method returns the requirements object.
 
+=head2 version_range_for_module
+
+  $req->version_range_for_module( $another_req_object );
+
 =head2 add_requirements
 
   $req->add_requirements( $another_req_object );
 
-This method adds all the requirements in the given CPAN::Meta::Requirements object
-to the requirements object on which it was called.  If there are any conflicts,
-an exception is thrown.
+This method adds all the requirements in the given CPAN::Meta::Requirements
+object to the requirements object on which it was called.  If there are any
+conflicts, an exception is thrown.
 
 This method returns the requirements object.
 
@@ -881,7 +627,17 @@ This returns a string containing the version requirements for a given module in
 the format described in L<CPAN::Meta::Spec> or undef if the given module has no
 requirements. This should only be used for informational purposes such as error
 messages and should not be interpreted or used for comparison (see
-L</accepts_module> instead.)
+L</accepts_module> instead).
+
+=head2 structured_requirements_for_module
+
+  $req->structured_requirements_for_module( $module );
+
+This returns a data structure containing the version requirements for a given
+module or undef if the given module has no requirements.  This should
+not be used for version checks (see L</accepts_module> instead).
+
+Added in version 2.134.
 
 =head2 required_modules
 
@@ -982,20 +738,23 @@ A version number without an operator is equivalent to specifying a minimum
 =head2 from_string_hash
 
   my $req = CPAN::Meta::Requirements->from_string_hash( \%hash );
+  my $req = CPAN::Meta::Requirements->from_string_hash( \%hash, \%opts );
 
-This is an alternate constructor for a CPAN::Meta::Requirements object.  It takes
-a hash of module names and version requirement strings and returns a new
-CPAN::Meta::Requirements object. As with add_string_requirement, a
-version can be a Perl "v-string".
+This is an alternate constructor for a CPAN::Meta::Requirements
+object. It takes a hash of module names and version requirement
+strings and returns a new CPAN::Meta::Requirements object. As with
+add_string_requirement, a version can be a Perl "v-string". Optionally,
+you can supply a hash-reference of options, exactly as with the L</new>
+method.
 
-=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
+=for :stopwords cpan testmatrix url bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
 
 =head1 SUPPORT
 
 =head2 Bugs / Feature Requests
 
 Please report any bugs or feature requests through the issue tracker
-at L<https://github.com/dagolden/CPAN-Meta-Requirements/issues>.
+at L<https://github.com/Perl-Toolchain-Gang/CPAN-Meta-Requirements/issues>.
 You will be notified automatically of any progress on your issue.
 
 =head2 Source Code
@@ -1003,9 +762,9 @@ You will be notified automatically of any progress on your issue.
 This is open source software.  The code repository is available for
 public review and contribution under the terms of the license.
 
-L<https://github.com/dagolden/CPAN-Meta-Requirements>
+L<https://github.com/Perl-Toolchain-Gang/CPAN-Meta-Requirements>
 
-  git clone https://github.com/dagolden/CPAN-Meta-Requirements.git
+  git clone https://github.com/Perl-Toolchain-Gang/CPAN-Meta-Requirements.git
 
 =head1 AUTHORS
 
@@ -1023,7 +782,7 @@ Ricardo Signes <rjbs@cpan.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Ed J Karen Etheridge robario
+=for stopwords Ed J Graham Knop Karen Etheridge Leon Timmermans Ricardo Signes robario Tatsuhiko Miyagawa
 
 =over 4
 
@@ -1033,12 +792,32 @@ Ed J <mohawk2@users.noreply.github.com>
 
 =item *
 
+Graham Knop <haarg@haarg.org>
+
+=item *
+
 Karen Etheridge <ether@cpan.org>
 
 =item *
 
+Leon Timmermans <fawaka@gmail.com>
+
+=item *
+
+Ricardo Signes <rjbs@semiotic.systems>
+
+=item *
+
 robario <webmaster@robario.com>
 
+=item *
+
+Tatsuhiko Miyagawa <miyagawa@bulknews.net>
+
+=item *
+
+Tatsuhiko Miyagawa <miyagawa@gmail.com>
+
 =back
 
 =head1 COPYRIGHT AND LICENSE
